@@ -17,7 +17,7 @@ use {
             program::id,
             state::{Authorized, Lockup},
         },
-        sysvar::{self, clock::Clock, rent::Rent, stake_history::StakeHistory},
+        sysvar::{clock::Clock, rent::Rent, stake_history::StakeHistory},
     },
 };
 
@@ -204,11 +204,11 @@ pub fn process_instruction(
                 .feature_set
                 .is_active(&feature_set::stake_program_v4::id())
             {
-                Some(invoke_context.get_sysvar::<Clock>(&sysvar::clock::id())?)
+                Some(invoke_context.get_sysvar_cache().get_clock()?)
             } else {
                 None
             };
-            me.set_lockup(&lockup, &signers, clock.as_ref())
+            me.set_lockup(&lockup, &signers, clock.as_deref())
         }
         StakeInstruction::InitializeChecked => {
             if invoke_context
@@ -326,8 +326,8 @@ pub fn process_instruction(
                     epoch: lockup_checked.epoch,
                     custodian,
                 };
-                let clock = Some(invoke_context.get_sysvar::<Clock>(&sysvar::clock::id())?);
-                me.set_lockup(&lockup, &signers, clock.as_ref())
+                let clock = Some(invoke_context.get_sysvar_cache().get_clock()?);
+                me.set_lockup(&lockup, &signers, clock.as_deref())
             } else {
                 Err(InstructionError::InvalidInstructionData)
             }
@@ -341,8 +341,11 @@ mod tests {
         super::*,
         crate::stake_state::{Meta, StakeState},
         bincode::serialize,
-        solana_program_runtime::invoke_context::{
-            mock_process_instruction, prepare_mock_invoke_context, InvokeContext,
+        solana_program_runtime::{
+            invoke_context::{
+                mock_process_instruction, prepare_mock_invoke_context, InvokeContext,
+            },
+            sysvar_cache::SysvarCache,
         },
         solana_sdk::{
             account::{self, AccountSharedData},
@@ -354,9 +357,9 @@ mod tests {
                 instruction::{self, LockupArgs},
                 state::{Authorized, Lockup, StakeAuthorize},
             },
-            sysvar::{stake_history::StakeHistory, Sysvar},
+            sysvar::{self, stake_history::StakeHistory},
         },
-        std::{cell::RefCell, rc::Rc, str::FromStr},
+        std::{borrow::Cow, cell::RefCell, rc::Rc, str::FromStr},
     };
 
     fn create_default_account() -> Rc<RefCell<AccountSharedData>> {
@@ -428,11 +431,10 @@ mod tests {
         let processor_account = AccountSharedData::new_ref(0, 0, &solana_sdk::native_loader::id());
         let program_indices = vec![preparation.accounts.len()];
         preparation.accounts.push((id(), processor_account));
-        let mut data = Vec::with_capacity(sysvar::clock::Clock::size_of());
-        bincode::serialize_into(&mut data, &sysvar::clock::Clock::default()).unwrap();
         let mut invoke_context = InvokeContext::new_mock(&preparation.accounts, &[]);
-        let sysvars = [(sysvar::clock::id(), data)];
-        invoke_context.sysvars = &sysvars;
+        let mut sysvar_cache = SysvarCache::default();
+        sysvar_cache.set_clock(Clock::default());
+        invoke_context.sysvar_cache = Cow::Owned(sysvar_cache);
         invoke_context.push(
             &preparation.message,
             &preparation.message.instructions()[0],
@@ -1074,11 +1076,10 @@ mod tests {
         let processor_account = AccountSharedData::new_ref(0, 0, &solana_sdk::native_loader::id());
         let program_indices = vec![preparation.accounts.len()];
         preparation.accounts.push((id(), processor_account));
-        let mut data = Vec::with_capacity(sysvar::clock::Clock::size_of());
-        bincode::serialize_into(&mut data, &sysvar::clock::Clock::default()).unwrap();
         let mut invoke_context = InvokeContext::new_mock(&preparation.accounts, &[]);
-        let sysvars = [(sysvar::clock::id(), data)];
-        invoke_context.sysvars = &sysvars;
+        let mut sysvar_cache = SysvarCache::default();
+        sysvar_cache.set_clock(Clock::default());
+        invoke_context.sysvar_cache = Cow::Owned(sysvar_cache);
         invoke_context
             .push(
                 &preparation.message,
